import requests
import os
import json


class TMDBInfo:
    BASE_URL = 'https://api.themoviedb.org/3/search/movie'
    IMAGE_BASE_URL = 'https://image.tmdb.org/t/p/w600'

    def __init__(self, title):
        """ :param title: The title of the film to search on TMDB. """
        self.movie_json = self.search_film(title)

    def search_film(self, title):
        """
        Search for a film in themoviedb.org.

        :param title: The title of the film.
        :return: A json with all the data of the movie found.
        :raises: Exception if no movie is found.
        """
        api_key = os.environ.get('API_KEY')
        resp = requests.get(
            self.BASE_URL + '?language={lang}&api_key={api_key}&query={query}'.format(
                lang='es-ES',
                api_key=api_key,
                query=title
            )
        )

        if resp.ok:
            load = json.loads(resp.text)
            total = load.get('total_results')
            if total == 0:
                raise Exception('No results')
            results = load.get('results')
            return results[0]
        else:
            raise Exception('Error {code}: {msg}'.format(code=resp.status_code, msg=resp.text))

    def _get_from_json(self, data):
        return self.movie_json.get(data)

    def get_title(self):
        return self._get_from_json('title')

    def get_overview(self):
        return self._get_from_json('overview')

    def get_image(self):
        return self.IMAGE_BASE_URL + self._get_from_json('poster_path')



import click

from .movie import Movie
from .scraper import Scraper


@click.group()
@click.version_option()
def cli():
    """ImaginEmail looks up for new films in the imaginbank webpage and register them in a DB.
    After that, you can also notify an specific email if you want."""
    pass


@cli.command('notify', short_help='Notify an email.')
@click.option('--email', default='daniseijo12@gmail.com',
              help='The email that will receive the notification.')
def notify(email):
    """This script send an email with new movies to an account provided."""
    Movie.notify_to_email(email)


@cli.command('check_web', short_help='Check for new movies.')
def check_web():
    """Check for new movies in the imaginbank web"""
    scraper = Scraper()
    scraper.check_and_save_new_movies()



import os

import click

from .connection import Connection
from .notification import EmailNotification
from .tmdb import TMDBInfo


class Movie:
    def __init__(self, title, image, info, purchase_info, purchase_link):
        self.title = title
        self.image = image
        self.info = info
        self.purchase_info = purchase_info
        self.purchase_link = purchase_link

        self.db_name = os.environ.get('DB_NAME')

    def register_movie(self):
        click.echo('Registering movie "%s" on database.' % self.title)
        with Connection(self.db_name) as conn:
            conn.insert_movie(self)

    def _get_tmdb_info(self):
        info = TMDBInfo(title=self.title)

        self.title = info.get_title()
        self.overview = info.get_overview()
        self.image = info.get_image()

    @staticmethod
    def exists_movie(movie_title):
        with Connection(os.environ.get('DB_NAME')) as conn:
            if conn.movie_exists(movie_title):
                return True
        return False

    @staticmethod
    def notify_to_email(to_email):
        movies = []
        with Connection(os.environ.get('DB_NAME')) as conn:
            query = conn.get_not_notified_movies()
            for row in query:
                m = Movie(row[1], row[2], row[3], row[4], row[5])
                movies.append(m)

        en = EmailNotification()

        en.send_email(to_email, movies)



import re

import click
import requests
from bs4 import BeautifulSoup

from .movie import Movie


class Scraper:
    BASE_URL = 'https://www.imaginbank.com'
    movies = []

    def check_and_save_new_movies(self):
        soup = self.get_soup_from_web('/descuentos/experiencias_es.html')

        for discount in soup.find_all("li", class_="descuento"):
            if not self.exists_movie(discount):
                detail_view = self.get_detail_view(discount)
                self.compose_and_save_movie(detail_view)

    @staticmethod
    def exists_movie(discount):
        title_str = discount.find(class_="informacion-top").h2.text
        return Movie.exists_movie(title_str)

    def compose_and_save_movie(self, detail_view):
        image_str = str(self.BASE_URL + detail_view.find(class_="imagen-descuento").img.get('src'))
        title_str = str(detail_view.find(class_="informacion-top").h1.text)

        event_info = detail_view.find(class_="informacion-bottom")
        event_info.p['class'] = 'lead'

        info_str = str(event_info.p)

        madrid_info = detail_view.find(class_="popup_informacion").find(string=re.compile("Madrid"))

        if not madrid_info:
            purchase_info_str = None
            purchase_link_str = None
        else:
            purchase_info_str = str(madrid_info.next_element)
            purchase_info_str = ' '.join(purchase_info_str.replace('\n\r', '').split())
            purchase_link_str = str(madrid_info.find_next('a').get('href'))

        movie = Movie(title_str, image_str, info_str, purchase_info_str, purchase_link_str)
        movie.register_movie()

    def get_detail_view(self, discount):
        detail_url = discount.find(title='Ir a la oferta').get('href')
        return self.get_soup_from_web(detail_url)

    def get_soup_from_web(self, web):
        click.echo('Parsing web: "%s".' % (self.BASE_URL + web))
        resp = requests.get(self.BASE_URL + web)
        if resp.ok:
            resp.encoding = 'utf-8'
            return BeautifulSoup(resp.text, 'html.parser')
        else:
            click.echo('Error parsing web.')
            raise Exception('Error {code}: {msg}'.format(code=resp.status_code, msg=resp.text))



import os
import smtplib
from email.mime.text import MIMEText

import click
from jinja2 import Environment, FileSystemLoader

from .connection import Connection


class EmailNotification:
    def __init__(self, template_dir='templates'):
        if os.path.isdir(template_dir):
            self.template_dir = template_dir
        else:
            self.template_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)), template_dir)
        self.env = Environment(loader=FileSystemLoader(self.template_dir))
        self.db_name = os.environ.get('DB_NAME')

    def _mail_render(self, data, template):
        template = template + ".html"
        text = self.env.get_template(template)
        msg = text.render(data)
        return msg

    def send_email(self, email_to, movies):
        if len(movies) == 0:
            click.echo('No new movies to notify.')
            return

        click.echo('Sending email to: %s.' % email_to)
        msg_str = self._mail_render({'movies': movies}, 'template')

        msg = MIMEText(msg_str, 'html')
        server = smtplib.SMTP('smtp.gmail.com', 587)

        msg['Subject'] = "Nuevo preestreno con ImaginBank"
        msg['From'] = os.environ.get('EMAIL')
        msg['To'] = email_to

        server.starttls()
        server.login(os.environ.get('EMAIL'), os.environ.get('EMAIL_PASSWORD'))
        server.send_message(msg)
        for movie in movies:
            click.echo('Send email with new movie "%s".' % movie.title)
            self._update_sent_email(movie)
        server.quit()

    def _update_sent_email(self, movie):
        with Connection(self.db_name) as conn:
            conn.email_sent(movie)



import io

from setuptools import setup


def load_requirements(filename):
    with io.open(filename, encoding='utf-8') as reqfile:
        return [line.strip() for line in reqfile if not line.startswith('#')]


setup(
    name='ImaginEmail',
    packages=['imaginemail'],
    version='1.0.2',
    description='ImaginEmail is a program that search on an specific imaginbank web'
                'and notify an user about new offers and films next to Madrid',
    author='Daniel Seijo',
    author_email='daniseijo12@gmail.com',
    url='https://github.com/daniseijo/imaginemail',
    download_url='https://github.com/daniseijo/imaginemail/archive/1.0.2.tar.gz',
    license='MIT',
    install_requires=load_requirements('requirements.txt'),
    entry_points={
        'console_scripts': ['imaginemail = imaginemail.imaginemail:cli'],
    },
    keywords=['scraper', 'imaginbank', 'email'],
)



import sqlite3


class Connection:
    def __init__(self, db_file):
        self.db = sqlite3.connect(db_file)
        self.cursor = self.db.cursor()
        self.create_movies_table()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.db.commit()
        self.db.close()

    def create_movies_table(self):
        self.cursor.execute(
            '''CREATE TABLE IF NOT EXISTS movies(id INTEGER PRIMARY KEY AUTOINCREMENT, 
                                                 title TEXT UNIQUE,
                                                 image TEXT,
                                                 info TEXT,
                                                 purchase_info TEXT,
                                                 purchase_link TEXT,
                                                 email_sent BOOLEAN)''')

    def insert_movie(self, movie):
        t = (None, movie.title, movie.image, movie.info, movie.purchase_info, movie.purchase_link, 'False',)
        if not self.movie_exists(movie.title):
            self.cursor.execute('''INSERT INTO movies VALUES (?,?,?,?,?,?,?)''', t)

    def movie_exists(self, movie_title):
        title_tuple = (movie_title,)
        query_answer = self.cursor.execute("SELECT count(*) FROM movies WHERE title=?", title_tuple)
        result = query_answer.fetchone()
        if result[0] == 0:
            return False
        return True

    def email_sent(self, movie):
        title_tuple = (movie.title,)
        self.cursor.execute("UPDATE movies SET email_sent='True' WHERE title=?", title_tuple)

    def get_not_notified_movies(self):
        return self.cursor.execute("SELECT * FROM movies WHERE email_sent='False' AND purchase_info NOT NULL")
